package com.airfrance.squalecommon.datatransfertobject.transform.stats;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;

import com.airfrance.squalecommon.datatransfertobject.stats.ApplicationStatsDTO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.access.UserAccessBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ApplicationBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditDateComparator;

/**
 * Transformation d'une application sous forme BO
 * en dto reprsentatif des statistiques niveau application
 */
public class ApplicationStatsTransformer {

    /**
     * @param pApplis les applications
     * @param pDaysForTerminatedAudit le nombre de jours max pour lesquels il doit y avoir au moins un audit
     * russi pour que l'application soit active
     * @param pDaysForAllAudits le nombre de jours dfini pour compter les audits
     * @return les statistiques de chaque application sous forme d'une liste de DTO
     */
    public static List bo2Dto(List pApplis, int pDaysForTerminatedAudit, int pDaysForAllAudits) {
        List stats = new ArrayList();
        for (int i = 0; i < pApplis.size(); i++) {
            stats.add(bo2Dto((ApplicationBO) pApplis.get(i), pDaysForTerminatedAudit, pDaysForAllAudits));
        }
        return stats;
    }

    /**
     * Transformation en DTO des statistiques d'une application
     * @param pApplicationBO l'application
     * @param pDaysForTerminatedAudit le nombre de jours max pour lesquels il doit y avoir au moins un audit
     * russi pour que l'application soit active
     * @param pDaysForAllAudits le nombre de jours dfini pour compter les audits
     * @return les statistiques sous forme DTO
     */
    public static ApplicationStatsDTO bo2Dto(ApplicationBO pApplicationBO, int pDaysForTerminatedAudit, int pDaysForAllAudits) {
        ApplicationStatsDTO stat = new ApplicationStatsDTO();
        // On tri les audits selon leur date d'excution
        List sortedAudits = new ArrayList(pApplicationBO.getAudits());
        Collections.sort(sortedAudits, new AuditDateComparator(true));
        AuditBO firstTerminatedAudit = getFirstTerminatedAudit(sortedAudits);
        Collections.reverse(sortedAudits);
        AuditBO lastExecutedAudit = getLastExecutedAudit(sortedAudits);
        // On calcule la date limite pour dfinir l'activation d'une application
        Calendar minDate = Calendar.getInstance();
        minDate.add(Calendar.DAY_OF_MONTH, -pDaysForTerminatedAudit);
        stat.setApplicationName(pApplicationBO.getName());
        stat.setActivatedApplication(hasTerminatedAuditSince(sortedAudits, pDaysForTerminatedAudit));
        if (null != firstTerminatedAudit) {
            stat.setFirstTerminatedAuditDate(firstTerminatedAudit.getDate());
        }
        if (pApplicationBO.getUserAccesses().size() > 0) {
            stat.setLastAccess(((UserAccessBO) pApplicationBO.getUserAccesses().get(0)).getDate());
        }
        if (null != lastExecutedAudit) {
            stat.setLastAuditDuration(lastExecutedAudit.getDuration());
            stat.setLastAuditIsTerminated(lastExecutedAudit.getStatus() == AuditBO.TERMINATED);
        }
        AuditBO lastFailedAudit = getLastAudit(sortedAudits, new int[]{AuditBO.FAILED, AuditBO.PARTIAL});
        if (null != lastFailedAudit) {
            stat.setLastFailedAuditDate(lastFailedAudit.getDate());
        }
        AuditBO lastTerminatedAudit = getLastAudit(sortedAudits, new int[]{AuditBO.TERMINATED});
        if (null != lastTerminatedAudit) {
            stat.setLastTerminatedAuditDate(lastTerminatedAudit.getDate());
        }
        stat.setNbAudits(getNbAuditsSince(sortedAudits, pDaysForAllAudits));
        stat.setNbPartialOrFaliedAudits(getNbPartialOrFailedAudits(sortedAudits));
        stat.setNbTerminatedAudits(getNbTerminatedAudits(sortedAudits));
        if (null != pApplicationBO.getServeurBO()) {
            // Peut tre nul dans le cas des applications non valides
            stat.setServerName(pApplicationBO.getServeurBO().getName());
        }
        stat.setValidatedApplication(pApplicationBO.getStatus() == ApplicationBO.VALIDATED);
        stat.setPurgeFrequency(pApplicationBO.getResultsStorageOptions());
        return stat;
    }

    /**
     * @param pAudits la liste trie des audits (ordre inverse)
     * @param pDaysForTerminatedAudit le nombre de jours max pour lesquels il doit y avoir au moins un audit
     * russi pour que l'application soit active
     * @return true si l'application a un audit russi dans le <code>pDaysForTerminatedAudit</code> derniers jours
     */
    private static boolean hasTerminatedAuditSince(List pAudits, int pDaysForTerminatedAudit) {
        boolean has = false;
        // On calcule la date limite pour compter les audits
        Calendar minDate = Calendar.getInstance();
        minDate.add(Calendar.DAY_OF_MONTH, -pDaysForTerminatedAudit);
        boolean dateInTime = true;
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size() && !has && dateInTime; i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            // On teste si on est encore dans la priode de temps souhaiter
            if (null != currentAudit.getDate() && currentAudit.getDate().compareTo(minDate.getTime()) < 0) {
                dateInTime = false;
            }
            has = (dateInTime && currentAudit.getStatus() == AuditBO.TERMINATED);
        }
        return has;
    }

    /**
     * @param pAudits la liste trie des audits (ordre inverse)
     * @return le dernier audit excut
     */
    private static AuditBO getLastExecutedAudit(List pAudits) {
        AuditBO lastAudit = null;
        // On parcours les audits jusqu' un trouv le dernier excut 
        // (i.e. le premier qui n'est pas en attente dans la liste)
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size() && null == lastAudit; i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            if (currentAudit.getStatus() != AuditBO.NOT_ATTEMPTED && currentAudit.getStatus() != AuditBO.RUNNING) {
                lastAudit = currentAudit;
            }
        }
        return lastAudit;
    }

    /**
     * @param pAudits les audits
     * @return ne nombre d'audits partiel ou en chec
     */
    private static int getNbPartialOrFailedAudits(List pAudits) {
        int nbAudits = 0;
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size(); i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            if (currentAudit.getStatus() == AuditBO.FAILED || currentAudit.getStatus() == AuditBO.PARTIAL) {
                // On veut le nombre d'audits en chec ou patiel
                nbAudits++;
            }
        }
        return nbAudits;
    }

    /**
     * @param pAudits les audits
     * @return ne nombre d'audits russis
     */
    private static int getNbTerminatedAudits(List pAudits) {
        int nbAudits = 0;
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size(); i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            if (currentAudit.getStatus() == AuditBO.TERMINATED) {
                // On veut le nombre d'audits en chec ou patiel
                nbAudits++;
            }
        }
        return nbAudits;
    }

    /**
     * Compte les audits dont le statut est "en echec", "partiel" ou "russi" raliss durant 
     * les <code>pDaysForAllAudits</code> derniers jours
     * @param pAudits les audits tris par date d'excution (ordre inverse)
     * @param pDaysForAllAudits le nombre de jours dfini pour compter les audits
     * @return le nombre d'audits raliss durant les <code>pDaysForAllAudits</code> derniers jours
     */
    private static int getNbAuditsSince(List pAudits, int pDaysForAllAudits) {
        int nbAudits = 0;
        // On calcule la date limite pour compter les audits
        Calendar minDate = Calendar.getInstance();
        minDate.add(Calendar.DAY_OF_MONTH, -pDaysForAllAudits);
        boolean dateInTime = true;
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size() && dateInTime; i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            // On teste si on est encore dans la priode de temps souhaiter
            if (null != currentAudit.getDate() && currentAudit.getDate().compareTo(minDate.getTime()) < 0) {
                dateInTime = false;
            }
            // On ne prend pas les status "en cours", "supprim" et "en attente"
            if (dateInTime && currentAudit.getStatus() != AuditBO.NOT_ATTEMPTED && currentAudit.getStatus() != AuditBO.DELETED && currentAudit.getStatus() != AuditBO.RUNNING) {
                // On veut le nombre d'audits excuts donc on prend tous les audits
                // sauf ceux en attente
                nbAudits++;
            }
        }
        return nbAudits;
    }

    /**
     * @param pAudits les audits tris par date d'excution (ordre inverse)
     * @param pStatus les statuts  rechercher
     * @return le dernier audit russi
     */
    private static AuditBO getLastAudit(List pAudits, int[] pStatus) {
        // Initialisation
        AuditBO lastTerminatedAudit = null;
        // On tri le tableau des status pour pouvoir utiliser la recherche
        Arrays.sort(pStatus);
        // On parcours les audits jusqu' un trouver un russi
        AuditBO currentAudit = null;
        int curStatus = 0;
        for (int i = 0; i < pAudits.size() && null == lastTerminatedAudit; i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            // On affecte le int pour flusher l'objet
            curStatus = currentAudit.getStatus();
            int id = Arrays.binarySearch(pStatus, curStatus);
            if (id >= 0 && id < pStatus.length) {
                lastTerminatedAudit = currentAudit;
            }
        }
        return lastTerminatedAudit;
    }

    /**
     * @param pAudits les audits tris par date d'excution
     * @return le premier audit russi
     */
    private static AuditBO getFirstTerminatedAudit(List pAudits) {
        AuditBO firstTerminatedAudit = null;
        // On parcours les audits jusqu' un trouv un russi
        AuditBO currentAudit = null;
        for (int i = 0; i < pAudits.size() && null == firstTerminatedAudit; i++) {
            currentAudit = (AuditBO) pAudits.get(i);
            if (currentAudit.getStatus() == AuditBO.TERMINATED) {
                firstTerminatedAudit = currentAudit;
            }
        }
        return firstTerminatedAudit;
    }
}
