package com.airfrance.squalix.core;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.bootstrap.initializer.Initializer;
import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafPersistenceException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.component.ApplicationDAOImpl;
import com.airfrance.squalecommon.daolayer.component.AuditDAOImpl;
import com.airfrance.squalecommon.daolayer.result.MarkDAOImpl;
import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.daolayer.result.rulechecking.RuleCheckingTransgressionDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ApplicationBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.roi.RoiMetricsBO;

/**
 * Calcule tous les ROI pour les audits termins
 * (Pour excuter le main sous unix mettre son nom dans le manifest.mf dans Main-Class)
 */
public class RoiMain {

    /**
     * Logger
     */
    private static Log mLOGGER = null;

    /**
     * Sauvegarde le roi calcul en base
     * @param pSession la session
     * @param pValue le nombre de correction
     * @param pApplication l'application courante
     * @param pAudit l'audit courant
     * @throws JrafDaoException si erreur
     */
    private static void createROI(ISession pSession, int pValue, ApplicationBO pApplication, AuditBO pAudit) throws JrafDaoException {
        RoiMetricsBO roi = new RoiMetricsBO();
        roi.setNbCorrections(pValue);
        roi.setComponent(pApplication);
        roi.setAudit(pAudit);
        // Initialisation du DAO
        MeasureDAOImpl roiDAO = MeasureDAOImpl.getInstance();
        roiDAO.create(pSession, roi);
    }

    /**
     * Calcule le nombre de corrections pour l'audit courant pour le ROI
     * @param pSession la session
     * @param pPreviousAudit l'audit prcdent
     * @param pCurrentAudit l'audit courant
     * @param pLogger le logger
     * @return le nombre de corrections faites entre les deux audits
     * @throws JrafDaoException si erreur
     */
    public static int calculateNbCorrection(ISession pSession, AuditBO pPreviousAudit, AuditBO pCurrentAudit, Log pLogger) throws JrafDaoException {
        int nbProgressions = 0;
        int nbSuppressions = 0;
        int transgressionDiff = 0;
        // En cas d'appel depuis une mthode extrieur
        if(mLOGGER == null){
            mLOGGER = pLogger;
        }
        /* On rcupre le nombre de corrections faites entre les deux audits */
        Long auditId = new Long(pCurrentAudit.getId());
        Long previousAuditId = new Long(pPreviousAudit.getId());
        MarkDAOImpl markDao = MarkDAOImpl.getInstance();
        // Les composants qui sont passs de zro  1, 2 ou 3
        nbProgressions = markDao.findCorrectionsWithProgessions(pSession, auditId, previousAuditId);
        mLOGGER.info("Nombre de progressions = " + nbProgressions);
        // Les composants qui taient  zro et qui n'existent plus
        nbSuppressions = markDao.findCorrectionsWithSuppressions(pSession, auditId, previousAuditId);
        mLOGGER.info("Nombre de suppressions = " + nbSuppressions);
        // La progression du nombre de transgressions de niveau erreur et warning
        RuleCheckingTransgressionDAOImpl ruleDao = RuleCheckingTransgressionDAOImpl.getInstance();
        int nbTransgressions = ruleDao.findNbErrorAndWarning(pSession, auditId);
        int nbPreviousTransgressions = ruleDao.findNbErrorAndWarning(pSession, previousAuditId);
        transgressionDiff = nbPreviousTransgressions - nbTransgressions;
        mLOGGER.info("Nombre de transgressions = " + transgressionDiff);
        if (transgressionDiff <= 0) {
            transgressionDiff = 0;
        }
        return nbProgressions + nbSuppressions + transgressionDiff;
    }

    /**
     * Calcul de tous les roi
     * @param pArgs arguments de lancement.
     * @throws JrafPersistenceException si erreur
     */
    public static void main(String pArgs[]) throws JrafPersistenceException {
        // lancement de linitialisation JRAF
        String rootPath = pArgs[0];
        String configFile = "/config/providers-config.xml";
        Initializer init = new Initializer(rootPath, configFile);
        init.initialize();
        // Maintenant que le socle JRAF est initialis, on peut crer un logger
        mLOGGER = LogFactory.getLog(RoiMain.class);
        ISession session = null;
        try {
            session = PersistenceHelper.getPersistenceProvider().getSession();
            //Instanciation des DAO
            AuditDAOImpl auditDAO = AuditDAOImpl.getInstance();
            ApplicationDAOImpl appliDAO = ApplicationDAOImpl.getInstance();
            MeasureDAOImpl measureDAO = MeasureDAOImpl.getInstance();
            // On rcupre tous les audits termins
            List audits = auditDAO.findWhereStatus(session, AuditBO.TERMINATED);
            // Pour chaque audit, on rcupre son prcdent et on calcule le nombre de corrections
            AuditBO currentAudit;
            ApplicationBO currentApplication;
            AuditBO previousAudit;
            Long auditId;
            for (int i = 0; i < audits.size(); i++) {
                currentAudit = (AuditBO) audits.get(i);
                auditId = new Long(currentAudit.getId());
                // On charge l'application associe
                currentApplication = appliDAO.loadByAuditId(session, auditId);
                // Si il n'y a pas de mesure de ROI dj calcule pour cette application 
                if (null == measureDAO.load(session, new Long(currentApplication.getId()), auditId, RoiMetricsBO.class)) {
                    // On rcupre l'audit prcdent
                    previousAudit = auditDAO.findPreviousAudit(session, currentAudit.getId(), currentAudit.getHistoricalDate(), currentApplication.getId(), null);
                    if (null != previousAudit) {
                        // On calcule le nombre de corrections ralis entre les deux audits
                        int nbCorrections = calculateNbCorrection(session, previousAudit, currentAudit,null);
                        mLOGGER.info(
                            "nbCorrections = "
                                + nbCorrections
                                + " pour l'application \""
                                + currentApplication.getName()
                                + "\" de l'audit n "
                                + previousAudit.getId()
                                + "  l'audit n "
                                + currentAudit.getId());
                        // Cration de la mesure 
                        createROI(session, nbCorrections, currentApplication, currentAudit);
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (null != session) {
                session.closeSession();
            }
        }
    }

}
