package com.airfrance.squalix.tools.compiling.java;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ParametersConstants;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.StringParameterBO;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.core.exception.ConfigurationException;
import com.airfrance.squalix.tools.compiling.MockCompilingTask;
import com.airfrance.squalix.tools.compiling.CompilingMessages;
import com.airfrance.squalix.tools.compiling.configuration.MockCompilingConf;
import com.airfrance.squalix.util.file.FileUtility;

/**
 * Met  jour les paramtres temporaires lis  la tche de compilation Java : <br/>
 * <ul>
 *  <li> CLASSES_DIR : le rpertoire racine contenant les .class du projet
 *  <li> CLASSPATH : le classpath du projet
 * </ul>
 */
public class JavaMockCompilingTask extends MockCompilingTask {
    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(JavaMockCompilingTask.class);

    /** configuration de la tche */
    private MockCompilingConf mConfiguration;

    /** Le classpath */
    private StringBuffer mClasspath;

    /**
     * Constructeur par dfaut
     */
    public JavaMockCompilingTask() {
        mName = "JavaMockCompilingTask";
        mConfiguration = new MockCompilingConf();
        mClasspath = new StringBuffer();
    }

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.core.AbstractTask#execute()
     */
    public void execute() throws TaskException {
        try {
            // on excute la partie commune
            super.execute();
            // on construit le classpath
            String userClasspath = mClasspath.toString();
            mClasspath = buildClasspath(userClasspath);
            // On l'ajoute aux paramtres temporaires
            mData.putData(TaskData.CLASSPATH, mClasspath.toString());
            LOGGER.debug("classpath = " + mClasspath.toString());
            // positionne les donnes sur la taille du file System
            affectFileSystemSize(mData.getData(TaskData.CLASSES_DIRS), true);
        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * Il faut parcourir tout le classpath afin de mettre les chemins en absolu
     * et vrifier qu'ils existent 
     * @param pUserClasspath le classpath que l'utilisateur a donn
     * @return le classpath bien construit et vrifi
     * @throws ConfigurationException si le classpath n'est pas bien construit
     */
    private StringBuffer buildClasspath(String pUserClasspath) throws ConfigurationException {
        // On rcupre le view path
        String viewPath = (String) mData.getData(TaskData.VIEW_PATH);
        if (viewPath == null) {
            String message = CompilingMessages.getString("exception.view_path.not_found");
            LOGGER.error(message);
            // Renvoi d'une exception de configuration
            throw new ConfigurationException(message);
        }
        StringBuffer classpath = new StringBuffer();
        // On spare les lments
        String[] elements = pUserClasspath.split(";");
        // Pour chaque lment, on construit son chemin absolu, on vrifie qu'il existe
        // et on construit le classpath
        for (int i = 0; i < elements.length; i++) {
            if (elements[i].length() > 0) {
                File file = new File(elements[i]);
                file = FileUtility.getAbsoluteFile(viewPath, file);
                if (file.exists()) {
                    classpath.append(file.getAbsolutePath());
                    classpath.append(";");
                } else {
                    // On log un warning un car cela peut perturber l'excution de mccabe
                    String message = CompilingMessages.getString("exception.element_does_not_exist", elements[i]);
                    LOGGER.warn(message);
                    initError(message);
                }
            }
        }
        /* on cre un buffer pour dfinir le chemin du dossier 
         * contenant les ressources ncessaires  la compilation */
        // On rcupre la version de java
        StringParameterBO versionParam = (StringParameterBO) mProject.getParameters().getParameters().get(ParametersConstants.DIALECT);
        if (versionParam == null) {
            String message = CompilingMessages.getString("java.exception.task.dialect_not_set");
            LOGGER.error(message);
            // Renvoi d'une exception de configuration
            throw new ConfigurationException(message);
        }
        StringBuffer path = new StringBuffer(CompilingMessages.getString("dir.root.java"));
        path.append("/");
        path.append(versionParam.getValue());
        path.append("/");
        addCompilingRessourcesToClasspath(classpath, new File(path.toString().replace('.', '_')));
        return classpath;
    }

    /** 
     * {@inheritDoc}
     * Place le paramtre temporaire <code>CLASSES_DIR</code>
     * @see com.airfrance.squalix.tools.compiling.MockCompilingTask#setCompiledDirInTempMap(java.util.List)
     */
    protected void setCompiledDirInTempMap(List pDirs) throws TaskException {
        try {
            // Rcupre le classpath
            StringParameterBO classpathParam = (StringParameterBO) mTaskParam.getParameters().get(ParametersConstants.CLASSPATH);
            if (classpathParam == null) {
                String message = CompilingMessages.getString("exception.classpath_not_set");
                LOGGER.error(message);
                // Renvoi d'une exception de configuration
                throw new ConfigurationException(message);
            }
            // On remplace les sparateurs unix par ceux de windows en cas
            // car c'est de cette faon qu'il est rcupr par la tche de compilation Java
            mClasspath = new StringBuffer(classpathParam.getValue().replace(':', ';'));
            if (mClasspath.length() > 0 && !mClasspath.toString().endsWith(";")) {
                mClasspath.append(";");
            }
            // On rcupre le chemin absolu du rpertoire qui va contenir
            // nos .class
            mConfiguration.parse(new FileInputStream("config/mockcompiling-config.xml"));
            // On le cre
            File root = new File(mConfiguration.getRootDirectory());
            root.mkdirs();
            // On copie tous les .class qui se trouvent dans la liste donne 
            // en paramtre dans le rpertoire racine
            for (int i = 0; i < pDirs.size(); i++) {
                StringParameterBO pathParam = (StringParameterBO) pDirs.get(i);
                String path = pathParam.getValue();
                // On enlve ce chemin du classpath car on va rajouter le rpertorie global
                mClasspath = new StringBuffer(mClasspath.toString().replaceAll(path, ""));
                // On rcupre le fichier si il existe et qu'il peut tre lu
                File compiledDir = verifyPath(new File(path));
                // On copie tous les .class de ce rpertoire (ou fichier compress)
                // dans le rpertoire de la configuration
                if(compiledDir.isDirectory()) {
                    // On copie tous les fichiers
                    File[] files = compiledDir.listFiles();
                    for(int j=0; j<files.length; j++) {
                        FileUtility.copyInto(files[j], root);
                    }
                } else {
                    FileUtility.copyAndExtractInto(compiledDir, root);
                }
            }
            // On ajoute le rpertoire au classpath
            mClasspath.append(root.getAbsolutePath());
            mClasspath.append(";");
            // On ajoute le paramtre  la map temporaire
            List classesDirs = new ArrayList();
            classesDirs.add(root.getAbsolutePath());
            mData.putData(TaskData.CLASSES_DIRS, classesDirs);

        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * @param pPath le chemin relatif ou absolu d'un rpertoire de fichiers compils
     * @return le rpertoire de fichiers compils si il existe et peut-tre lu
     * @throws ConfigurationException si erreur de configuration
     */
    private File verifyPath(File pPath) throws ConfigurationException {
        File result = null;
        if (pPath.isAbsolute() && pPath.exists()) {
            result = pPath;
        } else {
            // Le fichier est suppos tre relatif  la vue
            String viewPath = (String) getData().getData(TaskData.VIEW_PATH);
            if (viewPath == null) {
                String message = CompilingMessages.getString("exception.view_path.not_found");
                LOGGER.error(message);
                // Renvoi d'une exception de configuration
                throw new ConfigurationException(message);
            }
            result = new File(viewPath, pPath.getPath());
            if (!result.exists()) {
                String message = CompilingMessages.getString("error.file.not_found", result.getAbsolutePath());
                LOGGER.error(message);
                // Renvoi d'une exception de configuration
                throw new ConfigurationException(message);
            }
        }
        // Si on a pas le droit de lecture sur le fichier, on lance une exception
        if (!result.canRead()) {
            String message = CompilingMessages.getString("error.cannot_read_file", result.getAbsolutePath());
            LOGGER.error(message);
            // Renvoi d'une exception de configuration
            throw new ConfigurationException(message);
        }
        return result;
    }

    /**
     * Ajoute tous les .jar, .zip et .class au classpath contenu dans le rpertoire pass en paramtre
     * @param pClasspath le classpath
     * @param pRessource le fichier  ajouter ou le rpertoire  parcourir
     */
    private void addCompilingRessourcesToClasspath(StringBuffer pClasspath, File pRessource) {
        // Vrification du classpath
        if (pClasspath.length() > 0 && !pClasspath.toString().endsWith(";")) {
            pClasspath.append(";");
        }
        // On va rcuprer tous les .jar, .zip ou .classes
        if (null != pRessource && pRessource.isDirectory() && pRessource.canRead()) {
            File[] files = pRessource.listFiles(new ExtensionsFileFilter());
            for(int i=0; i<files.length; i++) {
                addCompilingRessourcesToClasspath(pClasspath, files[i]);
            }
        } else if (pRessource.exists()) {
            // On l'ajoute directement au classpath
            pClasspath.append(pRessource.getAbsolutePath());
        }
    }

    /**
     * Classe de filtre sur les extensions .jar, .zip et .class
     */
    private class ExtensionsFileFilter implements FileFilter {

        /** La liste des extensions */
        private List mExtensions;

        /**
         * Le constructeur par dfaut
         */
        public ExtensionsFileFilter() {
            mExtensions = new ArrayList();
            mExtensions.add("jar");
            mExtensions.add("zip");
            mExtensions.add("class");
        }

        /** 
         * {@inheritDoc}
         * @see java.io.FileFilter#accept(java.io.File)
         */
        public boolean accept(File pFile) {
            boolean result = pFile.isDirectory();
            if (pFile.isFile()) {
                // on rcupre l'extension
                String fileExtension = pFile.getName();
                fileExtension = fileExtension.substring(fileExtension.lastIndexOf(".") + 1, fileExtension.length());
                result = mExtensions.contains(fileExtension);
            }
            return result;
        }

    }
}
