package com.airfrance.squalix.tools.cpd;

import java.util.Iterator;

import net.sourceforge.pmd.cpd.Match;
import net.sourceforge.pmd.cpd.TokenEntry;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.daolayer.rulechecking.ProjectRuleSetDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.IntegerMetricBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.CpdTransgressionBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.RuleCheckingTransgressionBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.RuleCheckingTransgressionItemBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.ProjectRuleSetBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.RuleBO;

/**
 * Persistance des donnes de Cpd
 * Les donnes sont stockes dans une ruleset dynamique, ce ruleset contient une rgle
 * par langage
 */
public class CpdPersistor {
    /** Transgression */
    private CpdTransgressionBO mTransgression;
    /** RuleSet */
    private ProjectRuleSetBO mRuleSet;

    /** Prfixe de chemin */
    private String mPrefixPath;

    /**
     * Constructeur
     * @param pProjectBO projet
     * @param pAuditBO audit
     * @param pPrefixPath prfixe de path
     */
    public CpdPersistor(ProjectBO pProjectBO, AuditBO pAuditBO, String pPrefixPath) {
        mPrefixPath = pPrefixPath;
        mRuleSet = createRuleSet(pProjectBO);
        mTransgression = createTransgression(pProjectBO, pAuditBO);
    }

    /**
     * Cration de la transgression
     * @param pProjectBO projet
     * @param pAuditBO audit
     * @return transgression
     */
    protected CpdTransgressionBO createTransgression(ProjectBO pProjectBO, AuditBO pAuditBO) {
        // Cration de la transgression
        CpdTransgressionBO transgression = new CpdTransgressionBO();
        transgression.setAudit(pAuditBO);
        transgression.setComponent(pProjectBO);
        transgression.setRuleSet(mRuleSet);
        transgression.setTaskName("CpdTask");
        return transgression;
    }

    /**
     * Cration du RuleSet
     * @param pProject projet
     * @return rgle de copy/paste
     */
    protected ProjectRuleSetBO createRuleSet(ProjectBO pProject) {
        ProjectRuleSetBO ruleset = new ProjectRuleSetBO();
        ruleset.setProject(pProject);
        return ruleset;
    }

    /**
     * Cration d'une rgle
     * @param pLanguage langage
     * @return rgle
     */
    protected RuleBO createRule(String pLanguage) {
        RuleBO rule = new RuleBO();
        rule.setCategory("copypaste");
        rule.setCode("cpd." + pLanguage);
        rule.setSeverity("error");
        rule.setRuleSet(mRuleSet);
        mRuleSet.addRule(rule);
        return rule;
    }
    /**
     * Sauvegarde des rsultats
     * @param pSession session
     * @throws JrafDaoException si erreur
     */
    public void storeResults(ISession pSession) throws JrafDaoException {
        // Cration du RuleSet
        ProjectRuleSetDAOImpl.getInstance().create(pSession, mRuleSet);
        // Sauvegarde des donnes dans la base
        MeasureDAOImpl.getInstance().create(pSession, mTransgression);
    }

    /**
     * Ajout d'un rsultat
     * @param pLanguage langage
     * @param pMatches dtections
     */
    public void addResult(String pLanguage, Iterator pMatches) {
        RuleBO rule = createRule(pLanguage);
        int nbOcc = 0;
        int dupLinesNb = 0;
        // On traite chaque dtection
        while (pMatches.hasNext()) {
            Match m = (Match) pMatches.next();
            StringBuffer detail = new StringBuffer();
            detail.append(CpdMessages.getString("copypaste.summary", new Object[] { new Integer(m.getLineCount()), new Integer(m.getTokenCount())}));
            detail.append('\n');
            nbOcc++;
            Iterator occ = m.iterator();
            // Parcours de chaque occurrence de copy/paste
            String itemFileName = "";
            int itemLine = 0;
            while (occ.hasNext()) {
                TokenEntry entry = (TokenEntry) occ.next();
                String fileName = entry.getTokenSrcID();
                // On limine dans le nom du fichier le nom de la vue
                if (fileName.startsWith(mPrefixPath)) {
                    fileName = fileName.substring(mPrefixPath.length());
                }
                detail.append(CpdMessages.getString("copypaste.occurrenceentry", new Object[] { new Integer(entry.getBeginLine()), fileName }));
                detail.append('\n');
                // On affecte le premier fichier et la premire ligne trouve  l'item
                if(itemFileName.length() == 0) {
                    itemFileName = fileName;
                    itemLine = entry.getBeginLine();
                }
            }
            dupLinesNb += (m.getMarkCount() - 1) * m.getLineCount();
            // On ne fait persister que les 100 premiers items par rgle
            // ICI il n'y a qu'une rgle dfinie par langage
            if (nbOcc <= RuleCheckingTransgressionBO.MAX_DETAILS) {
                RuleCheckingTransgressionItemBO item = new RuleCheckingTransgressionItemBO();
                // Taille maximale pour les dtails
                final int MAX_LENGTH = 3000;
                // Troncature du message si besoin
                if (detail.length() > MAX_LENGTH) {
                    detail.setLength(MAX_LENGTH - 1);
                }
                item.setMessage(detail.toString());
                item.setRule(rule);
                item.setComponentFile(itemFileName);
                item.setLine(itemLine);
                mTransgression.getDetails().add(item);
            }
        }
        // On ajoute une mtrique de type Integer pour chaque rgle transgresse
        IntegerMetricBO metric = new IntegerMetricBO();
        metric.setName(rule.getCode());
        metric.setValue(nbOcc);
        metric.setMeasure(mTransgression);
        mTransgression.putMetric(metric);
        // Cration de la mtrique donnant le nombre total de lignes pour le langage
        mTransgression.setDuplicateLinesForLanguage(pLanguage, new Integer(dupLinesNb));
        // Incrment du nombre total de lignes dupliques
        mTransgression.incrementDuplicateLinesNumber(dupLinesNb);
    }
}
