package com.airfrance.squalix.tools.jdepend;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.regex.PatternSyntaxException;

import jdepend.framework.JDepend;
import jdepend.framework.JavaPackage;
import jdepend.framework.PackageFilter;

import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.PackageBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ListParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ParametersConstants;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.jdepend.JDependPackageMetricsBO;
import com.airfrance.squalix.core.AbstractTask;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.util.buildpath.BuildProjectPath;
import com.airfrance.squalix.util.file.FileUtility;
import com.airfrance.squalix.util.parser.JavaParser;
import com.airfrance.squalix.util.repository.ComponentRepository;

/**
 * Tche JDepend pour l'analyse des packages java
 */
public class JDependTask extends AbstractTask {

    /** Aide  la cration de composants */
    private ComponentRepository mRepository;

    /** l'outil jdepend */
    private JDepend mJDepend;

    /** le filtre pour construire les packages valides */
    private JDependFilter mFilter;

    /** l'ensemble des packages que l'on considre comme valides */
    private ArrayList mValidPackages;

    /** liste des sources  analyser (en tenant compte des exclusions et inclusions, 
     * (rcupres car rutilises) */
    private List mIncludedFileNames = new ArrayList(0);

    // Cette tache n'intervient pas dans le calcul de la taille du file system

    /**
     * Constructeur par defaut
     */
    public JDependTask() {
        mName = "JDependTask";
        mValidPackages = new ArrayList();
        mFilter = new JDependFilter();

    }

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.core.Task#execute()
     */
    public void execute() throws TaskException {
        try {
            // Le chemin racine du projet (vue clearcase, ...)
            String viewPath = (String) mData.getData(TaskData.VIEW_PATH);
            // la liste des sources
            List srcs = (List) ((ListParameterBO) mProject.getParameters().getParameters().get(ParametersConstants.SOURCES)).getParameters();

            // Les nom des fichiers qui peuvent tre persists
            // (on ne veut pour cette tche que les .java)
            mIncludedFileNames =
                FileUtility.getIncludedFiles(
                    viewPath,
                    BuildProjectPath.buildProjectPath(viewPath, srcs),
                    (ListParameterBO) mProject.getParameter(ParametersConstants.INCLUDED_PATTERNS),
                    (ListParameterBO) mProject.getParameter(ParametersConstants.EXCLUDED_PATTERNS),
                    (ListParameterBO) mProject.getParameter(ParametersConstants.EXCLUDED_DIRS),
                    new String[] { ".java" });
            // On va filtrer les packages de l'api java pour les exclure de l'analyse
            PackageFilter filter = new PackageFilter();
            filter.addPackage("java.*");
            filter.addPackage("javax.*");
            filter.addPackage("org.ietf.jgss");
            filter.addPackage("org.omg.*");
            filter.addPackage("org.w3c.*");
            filter.addPackage("org.xml.*");
            mJDepend = new JDepend(filter);
            mRepository = new ComponentRepository(mProject, getSession());
            // on rcupre les rpertoires o se trouvent les .class
            List classesDirs = (List) mData.getData(TaskData.CLASSES_DIRS);
            // on affecte ces rpertoires comme tant les rpertoires 
            // racine pour l'analyse
            for (int i = 0; i < classesDirs.size(); i++) {
                mJDepend.addDirectory((String) classesDirs.get(i));
            }
            // rcupre l'ensemble des packages dfinis pour l'arborescence dont
            // on a dfini la racine ci dessus
            Collection packages = mJDepend.analyze();
            Iterator iter = packages.iterator();
            JDependPackageMetricsBO measure;
            Collection measuresColl = new ArrayList(0);
            JavaParser parser = new JavaParser(mProject);
            // on cr d'abord la liste des packages  prendre en compte
            // en mettant la chaine vide comme parent root
            // on lance la construction des rpertoires valides
            for (int i = 0; i < classesDirs.size(); i++) {
                buildValidPackagesSet(new File((String) classesDirs.get(i)), "");
            }
            while (iter.hasNext()) {
                JavaPackage p = (JavaPackage) iter.next();
                if (mValidPackages.contains(p.getName())) {
                    measure = new JDependPackageMetricsBO();
                    // on affecte toutes les mesures
                    measure.setCa(p.afferentCoupling());
                    measure.setCe(p.efferentCoupling());
                    measure.setNumberOfClassesAndInterfaces(p.getClassCount());
                    // pour les metrics flottantes, 
                    // on prend des valeurs entires en pourcentage (x100)
                    final int TOINT = 100;
                    measure.setAbstractness(p.abstractness() * TOINT);
                    measure.setDistance(p.distance() * TOINT);
                    measure.setInstability(p.instability() * TOINT);
                    // gestion du cycle ventuel
                    measure.setHaveCycles(p.containsCycle());
                    List list = new ArrayList(0);
                    if (p.containsCycle()) {
                        // collecte les cycles et les mets dans la liste
                        p.collectCycle(list);
                        // pour faciliter l'affichage on a une mthode intermdiaire
                        // qui met en forme
                        measure.setCycle(getCycle(list));
                    }
                    // cre le composant et le fait persister
                    PackageBO packBO = parser.getPackage(p.getName());
                    measure.setComponent(mRepository.persisteComponent(packBO));
                    measure.setAudit(mAudit);
                    measure.setTaskName(mName);
                    measuresColl.add(measure);
                }
            }
            // persiste l'ensemble des mesures
            MeasureDAOImpl.getInstance().saveAll(getSession(), measuresColl);
        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * Mthode qui cre la liste des packages considrs comme valide,
     * c'est  dire qu'ils font partie de l'arborescence directe du rpertoire racine  analyser.
     * Par consquent on ne prend pas en compte les packages "java.*" ...
     * @param pRootFile le fichier racine de l'arborescence  parcourir
     * @param pParent le nom du package parent
     * @throws IOException en cas de problmes de lecture du fichier
     */
    private void buildValidPackagesSet(File pRootFile, String pParent) throws IOException {
        //pRootFile est forcment un rpertoire 
        // grace au filtre pour les appels rcursifs et par appel originel aussi
        if (pRootFile.canRead()) {
            // On rcupre tous les dossiers et les fichiers correspondant au pattern
            // des fichiers  analyser
            File[] list = pRootFile.listFiles(mFilter);
            // il faut vrifier l'appartenance aux sources meme si on sait que le parent prsum
            // appartenait aux sources car il y a des packages communs  plusieurs projets 
            // ("com" par exemple) et il faut etre sur que le parent du package courant 
            //  qui appartient aux sources
            // ou si il n'y a pas de rpertoire en dessous
            // ou si il y a pas que des repertoires en dessous
            // et que le repertoire courant fait partie des sources
            if (((list.length == 0 || (list.length != 0 && pRootFile.listFiles().length != 0)) && isMemberOfParameterList(mIncludedFileNames, pParent.replace('.', '/')))) {
                //  on ajoute l'ensemble des rpertoires de l'arborescence
                mValidPackages.add(pParent);
            }
            String filename = null;
            for (int i = 0; i < list.length; i++) {
                // pour mmoriser le parent courant afin de le conserver
                // pour faire l'appel rcursif sur chaque enfant 
                String parentMemory = pParent;
                filename = list[i].getName();
                File f = new File(filename);
                // il ne faut pas ajouter un "." au dbut
                if (!pParent.equals("")) {
                    pParent += "." + filename;
                } else {
                    pParent = filename;
                }
                // et on recommence sur chaque fils
                buildValidPackagesSet(list[i], pParent);
                // rattribue la bonne valeur
                pParent = parentMemory;

            }
        }
    }

    /**
     * indique si le fichier ou le rpertoire dont le nom est pSuffix 
     * appartient  l'ensemble des sources ou des rpertoires exclus 
     * (suivant la liste passe en argument) en effectuant une recherche rcursive.
     * @param pSuffix le nom du package transform dont les "." ont t remplacs par des "/"
     * @param pList la liste des noms absolus des fichiers
     * le cas d'utilisation
     * @return un boolen correspondant
     */
    private boolean isMemberOfParameterList(List pList, String pSuffix) {
        boolean result = false;
        for (int i = 0; !result && !result && i < pList.size(); i++) {
            // src ne peut tre que le nom d'un fichier
            String src = (String) pList.get(i);
            // on teste l'appartenance  la fois des rpertoires ou des fichiers
            try {
                if (!pSuffix.equals("") && src.matches(".*/" + pSuffix + "/.*\\..*")) {
                    // on a trouv le fichier ou le rpertoire dans l'arborescence
                    result = true;
                }
            } catch (PatternSyntaxException pse) {
                // Il ne peut donc pas s'agir d'un package
                result = false;
            }
        }
        return result;
    }

    /**
     * @return l'outil jDepend utilis
     */
    public JDepend getJDepend() {
        return mJDepend;
    }

    /**
     * @param pJDepend le nouveau JDepend 
     */
    public void setJDepend(JDepend pJDepend) {
        mJDepend = pJDepend;
    }

    /**
     * @param pList la liste des packages
     * @return le cycle sous une forme affichable
     */
    private String getCycle(List pList) {
        final int MAX_LEN = 2000;
        String result = "";
        // le package qui est la cause du cycle
        String cycleName = ((JavaPackage) pList.get(pList.size() - 1)).getName();
        for (int i = 0; i < pList.size() && result.length() < MAX_LEN; i++) {
            JavaPackage pkg = (JavaPackage) pList.get(i);
            String name = pkg.getName();
            // juste de la mise en forme d'affichage
            // suivant la faon dont JDepend rcupre les informations
            // cf source JDepend pour la mise en forme 
            if (cycleName.equals(name)) {
                result += "|->" + name + "\n";
            } else {
                result += "|  " + name + "\n";
            }
        }
        // retourne le cycle sous une forme affichable sur le web
        if (result.length() >= MAX_LEN) {
            result += "\n...";
        }
        return result;
    }

}