package com.airfrance.squalix.tools.ruleschecking;

import java.io.File;
import java.io.FilenameFilter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildEvent;
import org.apache.tools.ant.BuildListener;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Commandline.Argument;

import com.airfrance.squalix.core.exception.ConfigurationException;

/**
 * Processus checkstyle
 * L'excution de l'outil Checkstyle se fait par
 * l'intermdiaire de cette classe. Elle excute checkstyle
 * dans une tche ant de type java pour permettre l'accs
 * aux jars spcifique de checkstyle qui sont en conflit
 * avec ceux de hibernate.
 * Le processus prend en entre le fichier de configuration
 * au format xml ainsi que la liste des rpertoires contenant
 * les fichiers  analyser. Il fournit en sortie un fichier
 * au format XML qui contient les trasngressions repres
 */
public class CheckStyleProcess implements BuildListener {
    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CheckStyleProcess.class);
    /** Rpertoire des jars */
    private File mJarDirectory;
    /** Rpertoire des rapports */
    private File mReportDirectory;
    /** Indicateur d'erreur d'excution */
    private boolean mErrorOccurred;
    /** version des sources java */
    private String mJavaVersion;

    /**
     * Constructeur
     * @param pJarDirectory rpertoire des jars
     * @param pReportDirectory rpertoire des rapports
     * @param pJavaVersion la version des sources
     * @throws ConfigurationException si erreur
     */
    public CheckStyleProcess(File pJarDirectory, File pReportDirectory, String pJavaVersion) throws ConfigurationException {
        checkDirectory(pJarDirectory);
        mJarDirectory = pJarDirectory;
        checkDirectory(pReportDirectory);
        mReportDirectory = pReportDirectory;
        mJavaVersion = pJavaVersion;
    }

    /**
     * Vrification d'un rpertoire
     * @param pDirectory rpertoire
     * @throws ConfigurationException si rpertoire inexistant
     */
    private void checkDirectory(File pDirectory) throws ConfigurationException {
        if (!pDirectory.exists() || !pDirectory.isDirectory()) {
            // On essaye de le crer sinon on retourne une exception
            if (!pDirectory.mkdirs()) {
                throw new ConfigurationException(RulesCheckingMessages.getString("error.bad_directory", pDirectory));
            }
        }
    }
    /**
     * Analyse des sources avec Checkstyle
     * @param pRuleFile fichier de rgles
     * @param pSources rpertoires source
     * @param pReportName nom du rapport gnr
     * @return fichier gnr
     */
    public File analyseSources(File pRuleFile, String[] pSources, String pReportName) {
        File result = new File(mReportDirectory, pReportName);
        Java task = createAntTask(pRuleFile, pSources, result);
        mErrorOccurred = false;
        task.execute();
        // mErrorOccured peut passer  true
        return result;
    }

    /**
     * Indique si une erreur s'est produite
     * @return true si une erreur s'est produite
     */
    public boolean hasErrorOccurred() {
        return mErrorOccurred;
    }

    /**
     * Cration de la tche ANT
     * @param pRuleFile fichier de rgles
     * @param pSources sources  analyser
     * @param pResultFile fichier donnant le rsultat de l'analyse
     * @return tche ANT
     */
    private Java createAntTask(File pRuleFile, String[] pSources, File pResultFile) {
        Java task = new Java();
        Project antProject = new Project();
        antProject.addBuildListener(this);
        task.setProject(antProject);
        // On fork pour viter les problmes de classpath
        // lis  la version 2.5 de checkstyle qui utilise une version
        // non compatible de ANTLR dans le contexte hibernate
        task.setFork(true);
        task.setClassname("com.puppycrawl.tools.checkstyle.Main");
        // Cration du classpath
        Path path = createClassPath(antProject);
        task.setClasspath(path);
        Argument arg = task.createArg();
        // Augmentation de la mmoire alloue  la JVM
        Argument jvmArg = task.createJvmarg();
        jvmArg.setValue("-Xmx128M");
        jvmArg = task.createJvmarg();
        jvmArg.setValue("-Xss1M");
        arg = task.createArg();
        arg.setValue("-f");
        arg = task.createArg();
        arg.setValue("xml");
        arg = task.createArg();
        arg.setValue("-o");
        arg = task.createArg();
        arg.setValue(pResultFile.getAbsolutePath());
        arg = task.createArg();
        arg.setValue("-c");
        arg = task.createArg();
        arg.setValue(pRuleFile.getAbsolutePath());
        // Passage en revue des diffrents rpertoires source
        for (int i = 0; i < pSources.length; i++) {
            arg = task.createArg();
            arg.setValue("-r");
            arg = task.createArg();
            arg.setValue(pSources[i]);
        }
        return task;
    }

    /**
     * @param pAntProject projet ANT
     * @return classpath cr
     */
    private Path createClassPath(Project pAntProject) {
        Path path = new Path(pAntProject);
        // On cherche tous les fichiers jar prsents dans le rpertoire adquat
        File[] jarFiles = mJarDirectory.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.endsWith(".jar");
            }
        });
        for (int i = 0; i < jarFiles.length; i++) {
            path.setPath(jarFiles[i].getAbsolutePath());
        }
        // On ajoute le j2ee.jar en fonction de la version de java
        /* on cre un buffer pour dfinir le chemin du dossier 
         * contenant les ressources ncessaires  la compilation 
         */
        StringBuffer j2ee = new StringBuffer(RulesCheckingMessages.getString("dir.root.java"));
        j2ee.append("/");
        j2ee.append(mJavaVersion);
        j2ee.append("/");
        /* on cre le descripteur de fichier et on ajoute au path */
        File f = new File(j2ee.toString().replace('.', '_') + "j2ee.jar");
        path.setPath(f.getAbsolutePath());
        return path;
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#buildStarted(org.apache.tools.ant.BuildEvent)
     */
    public void buildStarted(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#buildFinished(org.apache.tools.ant.BuildEvent)
     */
    public void buildFinished(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#targetStarted(org.apache.tools.ant.BuildEvent)
     */
    public void targetStarted(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#targetFinished(org.apache.tools.ant.BuildEvent)
     */
    public void targetFinished(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#taskStarted(org.apache.tools.ant.BuildEvent)
     */
    public void taskStarted(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#taskFinished(org.apache.tools.ant.BuildEvent)
     */
    public void taskFinished(BuildEvent event) {
        log(event);
    }

    /** (non-Javadoc)
     * @see org.apache.tools.ant.BuildListener#messageLogged(org.apache.tools.ant.BuildEvent)
     */
    public void messageLogged(BuildEvent event) {
        log(event);
    }

    /**
     * @param pEvent vnement
     */
    private void log(BuildEvent pEvent) {
        // On adapte les logs en fonction de la gravit du log initial
        String message = pEvent.getMessage();
        Throwable exception = pEvent.getException();
        // Utilisation d'un switch car on ne peut pas
        // se passer du mapping entre en entier et une mthode
        //  appeler : on pourrait mmoriser dans une map
        // la mthode et l'entier mais cel est un peu lourd
        switch (pEvent.getPriority()) {
            case Project.MSG_ERR :
                LOGGER.error(message, exception);
                mErrorOccurred = true;
                break;

            case Project.MSG_WARN :
                LOGGER.warn(message, exception);
                break;

            case Project.MSG_INFO :
                LOGGER.info(message, exception);
                break;

            case Project.MSG_VERBOSE :
                LOGGER.trace(message, exception);
                break;

            case Project.MSG_DEBUG :
                LOGGER.debug(message, exception);
                break;

            default :
                // Par dfaut on log sur le flux de warning
                LOGGER.warn(message, exception);
                // par dfaut
                break;
        }
    }

}
